/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
#include "nsISupports.idl"

interface nsIAbCard;

[scriptable, uuid(eafe2488-4efb-4ac8-a6b4-7756eb1650a3)]
interface nsIAbDirSearchListener : nsISupports {
  void onSearchFinished(in long aResult, in AString aErrorMsg);

  void onSearchFoundCard(in nsIAbCard aCard);
};
