/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

@namespace html url("http://www.w3.org/1999/xhtml");

/*
 * This file's style only applies to broken objects, not the rest
 * of the page.
 */

.mainBox {
  font: message-box;
  font-size: 12px;
  text-align: center;
  display: table;
  width: 100%;
  height: 100%;
  background-image: repeating-linear-gradient(-45deg,
                                              rgba(65, 65, 65, 0.8),
                                              rgba(65, 65, 65, 0.8) 20px,
                                              rgba(69, 69, 69, 0.8) 20px,
                                              rgba(69, 69, 69, 0.8) 40px);
  color: white;
  text-shadow: 0 0 3.5px rgba(0, 0, 0, 0.8);
  border-radius: 6px;
  /* recessed effect with dark inner shadow and lightened bottom */
  box-shadow: inset 0 1px 3.5px rgba(0,0,0,0.8),
              0 1px 0 rgba(255,255,255,0.2);
  -moz-user-select: none;
  position: relative;
}

.hoverBox {
  display: table-cell;
  -moz-box-sizing: border-box;
  padding: 5px;
  vertical-align: middle;
  width: 100%;
  height: 100%;
}

html|a {
  color: #FFFFFF;
}

.icon {
  width: 48px;
  height: 48px;
  background-position: center;
  background-repeat: no-repeat;
  border-style: none;
  background-color: transparent;
  -moz-user-focus: ignore;
  margin-bottom: 6px;
}

:-moz-type-unsupported .icon,
:-moz-type-unsupported-platform .icon {
  background-image: url(chrome://mozapps/skin/plugins/contentPluginMissing.png);
}

:-moz-type-unsupported .icon[status="ready"] {
  background-image: url(chrome://mozapps/skin/plugins/contentPluginDownload.png);
}

:-moz-handler-clicktoplay .icon,
:-moz-handler-vulnerable-updatable .icon,
:-moz-handler-vulnerable-no-update .icon {
  background-image: url(chrome://mozapps/skin/plugins/contentPluginClickToPlay.png);
}

:-moz-handler-disabled .icon {
  background-image: url(chrome://mozapps/skin/plugins/contentPluginDisabled.png);
}

:-moz-handler-blocked .icon {
  background-image: url(chrome://mozapps/skin/plugins/contentPluginBlocked.png);
}

:-moz-handler-crashed .icon {
  background-image: url(chrome://mozapps/skin/plugins/contentPluginCrashed.png);
}

.throbber {
  padding-left: 16px; /* width of the background image */
  background: url("chrome://communicator/skin/icons/loading.gif") no-repeat;
  margin-left: 5px;
}

.msg {
  cursor: default;
}

:-moz-handler-clicktoplay .mainBox,
:-moz-handler-clicktoplay .icon,
:-moz-handler-clicktoplay .msgClickToPlay,
:-moz-handler-vulnerable-updatable .msgClickToPlay,
:-moz-handler-vulnerable-no-update .msgClickToPlay {
  cursor: pointer;
}

:-moz-handler-clicktoplay .msgTapToPlay {
  display: none;
}

.submitStatus div {
  min-height: 19px; /* height of biggest line (with throbber) */
}

.submitComment {
  width: 340px;
  height: 70px;
  padding: 5px;
  border: none;
  border-radius: 5px;
  resize: none;
  font-family: inherit;
  font-size: inherit;
}

.submitURLOptInBox {
  text-align: start;
}

.submitURLOptIn {
  margin-left: -1px;
}

.mainBox[chromedir="rtl"] .submitURLOptIn {
  margin-left: 0;
  margin-right: -1px;
}

.submitButtonBox {
  margin-top: 7px;
}

.submitButton {
  float: right;
}

.mainBox[chromedir="rtl"] .submitButton {
  float: left;
}


.helpIcon {
  float: left;
  display: inline-block;
  min-width: 16px;
  min-height: 16px;
  background: url(chrome://mozapps/skin/plugins/pluginHelp-16.png) no-repeat;
  cursor: pointer;
}

.mainBox[chromedir="rtl"] .helpIcon {
  float: right;
}

.closeIcon {
  display: block;
  position: absolute;
  width: 15px;
  height: 16px;
  top: 4px;
  right: 4px;
  border-style: none;
  background-color: transparent;
  background-image: url(chrome://global/skin/icons/close.gif);
  background-repeat: no-repeat;
}

.mainBox[chromedir="rtl"] .closeIcon {
  right: auto;
  left: 4px;
}

.closeIcon:focus,
.closeIcon:hover {
  background-image: url(chrome://global/skin/icons/close-hov.gif);
}

.closeIcon:active {
  background-image: url(chrome://global/skin/icons/close-act.gif);
}

.closeIcon::-moz-focus-inner {
  padding: 0px;
  border: 0px none;
}
