/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

/* ===== msgCompSMIMEOverlay.css ========================================
  == Styles for the S/Mime in composer window.
  ======================================================================= */

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");

#button-security {
  list-style-image: url("chrome://messenger/skin/smime/icons/smbtn1.gif");
  -moz-image-region: rect(0px 49px 33px 0px);
}

#button-security:hover {
  -moz-image-region: rect(0px 99px 33px 50px);
} 

#button-security:hover:active {
  -moz-image-region: rect(0px 149px 33px 100px);
} 

#button-security[disabled] {
  -moz-image-region: rect(0px 199px 33px 150px) !important;
} 


#msgcomposeWindow #signing-status {
  list-style-image: none;
  visibility: collapse;
}

#msgcomposeWindow[signing="ok"] #signing-status {
  list-style-image: url("chrome://messenger/skin/smime/icons/sbSignOk.gif");
  visibility: visible;
}

#msgcomposeWindow[signing="notok"] #signing-status {
  list-style-image: url("chrome://messenger/skin/smime/icons/sbSignNotOk.gif");
  visibility: visible;
}


#msgcomposeWindow #encryption-status {
  list-style-image: none;
  visibility: collapse;
}

#msgcomposeWindow[crypto="ok"] #encryption-status {
  list-style-image: url("chrome://messenger/skin/smime/icons/sbCryptoOk.gif");
  visibility: visible;
}

#msgcomposeWindow[crypto="notok"] #encryption-status {
  list-style-image: url("chrome://messenger/skin/smime/icons/sbCryptoNotOk.gif");
  visibility: visible;
}
